/* CSS for skin DARK */
/* Shipped with LemonLDAP::NG */

/* Use buttons CSS */
@import url("../../common/buttons.css");

/* Use input icons CSS */
@import url("../../common/inputicons.css");

/* Structure */
body {
background:#000;
color:#fff;
font-family:sans-serif;
font-weight:bold;
font-size:9pt;
margin:0;
padding:0;
}

/* Lists */
ul {
list-style:none;
margin:0;
padding:5px;
}

li {
list-style:none;
}

/* Links */
a img,:link img,:visited img {
border:none;
}

a, a:link, a:visited {
text-decoration:none;
color:orange;
}

/* Texts */
h3 {
margin:10px 0;
padding: 0 10px;
opacity:1;
font-size:12pt;
border-left: orange solid 20px;
text-align: left;
}

/* Tables */
table {
border:0;
margin:0;
}

th {
text-align:right;
border-bottom:orange solid 2px;
font-size:9pt;
padding:5px;
}

/* Forms */
input {
border:1px solid #CCC;
padding:5px;
width: 200px;
}

form input[name=openid_identifier] {
width: 300px;
}

/* All contents */
#logincontent, #mailcontent, #errorcontent, #menucontent, #notifcontent {
margin:100px auto 20px auto;
background:#4B4B4B;
padding:10px;
width:800px;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

#logincontent, #mailcontent, #errorcontent, #menucontent {
min-height:400px;
}

#logincontent form, #mailcontent form, #notifcontent form {
background:#E4E4E4;
opacity:0.9;
margin: 20px 20px 20px 250px;
padding:10px;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
color:#000;
min-height:200px;
height:auto;
}

#authMenu form {
margin: 20px;
}

#mailcontent form {
height:200px;
}

#mailcontent div.link {
margin:20px 0 0 250px;
}

div.loginlogo {
background:url(../images/lock.png) no-repeat center;
width:200px;
height:200px;
margin:40px 10px;
float:left;
}

#authMenu {
background:#E4E4E4;
opacity:0.9;
margin: 20px 20px 20px 250px;
text-align:center;
padding:5px;
font-size:9pt;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

#authMenu .ui-tabs-nav li a {
color:#CCC;
}

#authMenu li.ui-tabs-active a {
color:#000;
}

tr.authLogo {
text-align:center;
}

/* Footer */
#footer {
text-align:center;
font-weight:normal;
clear:both;
}

/* Logout and error */
#logout .buttons a, #logout .buttons button, #error .buttons a, #error .buttons button {
display:block;
display:inline !important;
float:none;
margin:0;
width:150px;
}

#logincontent #logout, #errorcontent #error {
margin-top:60px;
}

/* Menu */
#menucontent {
margin:20px auto;
width:80%;
height:auto;
}

#menucontent #logout {
text-align:center;
padding:10px 20px 20px 20px;
background:#E4E4E4;
opacity:0.9;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
color:#000;
margin:20px auto;
width:400px;
}

#menucontent #password, #menucontent #loginHistory {
text-align:center;
padding:10px 20px 20px 20px;
background:#E4E4E4;
opacity:0.9;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
color:#000;
margin:20px auto;
width:600px;
}

#menucontent div.user {
text-align:right;
margin:10px 20px 0 0;
}

#menucontent .ui-tabs-nav {
text-align:center;
background:#E4E4E4;
padding:5px;
font-size:9pt;
opacity:0.9;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
margin:10px auto;
width:600px;
}

#menucontent .ui-tabs-nav li a {
color:#CCC;
}

#menucontent li.ui-tabs-active a {
color:#000;
}

/* Message box */
div.message {
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

div.message li {
background-repeat:no-repeat;
background-position:5px 0;
padding:3px 0 3px 40px;
}

.message.negative li {
background-image:url("../../common/warning.png");
}

.message.positive li {
background-image:url("../../common/ok.png");
}

.message.warning li {
background-image:url("../../common/warning_triangle.png");
}

/* JQuery tabs */
.ui-tabs-hide {
display:none;
}

.ui-tabs-nav {
text-align:left;
margin:2px 0;
padding:6px 2px 5px 2px;
}

.ui-tabs-nav li{
display:inline-block;
margin-left:10px;
margin-top:5px;
margin-bottom:5px;
}

.ui-tabs-nav li a{
}

li.ui-tabs-active{
}

/* Applications list */
.clearfix {
clear:both;
}

div.category {
background:#E4E4E4;
opacity:0.9;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
color:#000;
margin:10px 50px;
padding:10px 20px;
clear:both;
}

div.category h3.catname {
background:#4B4B4B;
color:#FFF;
padding:5px 20px;
font-size:10pt;
}

div.application {
margin:10px;
padding:5px;
float:left;
width:250px;
height:30px;
}

div.application:hover {
background:#eee;
}

div.application img.applogo {
width:32px;
height:32px;
float:left;
}

div.application h4.appname {
text-align:left;
padding:0 5px;
margin:0 0 0 40px;
}

div.application p.appdesc {
text-align:left;
padding:0 5px;
margin:0 0 0 40px;
text-size:8pt;
color:#aaa;
font-style:italic;
}
