<TMPL_INCLUDE NAME="mail_header.tpl">

<p>
<lang en="Hello" fr="Bonjour" /> $cn,<br />
<br />
<lang en="Your new password is" fr="Votre nouveau mot de passe est" /> 
<span><img src="cid:key:skins/common/key.png" /></span>
<b>$password</b>
</p>

<TMPL_INCLUDE NAME="mail_footer.tpl">
