/* BUTTONS */
/* See http://particletree.com/features/rediscovering-the-button-element/ */

.buttons {
margin:0;
}

form .buttons {
margin:10px 0 0 0;
}

.buttons a, .buttons button {
display:block;
float:right;
margin:0 0 0 7px;
background-color:#f5f5f5;
border:1px solid #dedede;
border-top:1px solid #eee;
border-left:1px solid #eee;
text-decoration:none;
font-weight:bold;
color:#565656;
cursor:pointer;
padding:5px 10px 6px 7px; /* Links */
}

.buttons button {
width:auto;
overflow:visible;
padding:4px 10px 3px 7px; /* IE6 */
}

.buttons button[type]{
padding:5px 10px 5px 7px; /* Firefox */
line-height:17px; /* Safari */
}

*:first-child+html button[type]{
padding:4px 10px 3px 7px; /* IE7 */
}

.buttons button img, .buttons a img{
margin:0 3px -3px 0 !important;
padding:0;
border:none;
width:16px;
height:16px;
}

/* STANDARD */

button:hover, .buttons a:hover, div.message{
background-color:#dff4ff;
border:1px solid #c2e1ef;
color:#336699;
}

.buttons a:active{
background-color:#6299c5;
border:1px solid #6299c5;
color:#fff;
}

/* POSITIVE */
button.positive, .buttons a.positive{
color:#529214;
}

.buttons a.positive:hover, button.positive:hover, div.positive{
background-color:#E6EFC2;
border:1px solid #C6D880;
color:#529214;
}

.buttons a.positive:active{
background-color:#529214;
border:1px solid #529214;
color:#fff;
}

/* NEGATIVE */
.buttons a.negative, button.negative{
color:#d12f19;
}

.buttons a.negative:hover, button.negative:hover, div.negative{
background:#fbe3e4;
border:1px solid #fbc2c4;
color:#d12f19;
}

.buttons a.negative:active{
background-color:#d12f19;
border:1px solid #d12f19;
color:#fff;
}

/* WARNING */
div.warning{
background-color:#ffe17a; 
border:1px solid #ffca10;
color:#a03400;
}

