/* CSS for skin IMPACT */
/* Shipped with LemonLDAP::NG */

/* -------------------------------------------------------------------------- */
/* Common                                                                     */
/* -------------------------------------------------------------------------- */


body {
  background:#F0F0F0 url("../images/bg.png") repeat-x;
  font-size: 0.8em;
  font-family:Arial, Helvetica, Verdana;
}

a {
  color: #0AAF11;
  text-decoration: none;
}

form {
  margin: 0px;
  padding: 0px;
}

hr.solid {
  margin-top: 15px;
  margin-bottom: 15px;
  border: 0px;
  height: 1px;
  color: #aaa;
  background: #aaa;
}

form input[name=user] {
  border: 1px solid #aaa;
  padding: 5px;
  width: 200px;
  background: #FFFFFF url("../../common/user_suit.png") 5px 5px no-repeat;
  padding-left: 25px;
}

form input[type=password], form input[name=captcha_user_code] {
  border: 1px solid #aaa;
  padding: 5px;
  width: 200px;
  background: #FFFFFF url("../../common/key.png") 5px 5px no-repeat;
  padding-left: 25px;
}

form input[name=mail] {
  border: 1px solid #aaa;
  padding: 5px;
  width: 200px;
  background: #FFFFFF url("../../common/email.png") 5px 5px no-repeat;
  padding-left: 25px;
}

form input[name=openid_identifier] {
  border: 1px solid #aaa;
  padding: 5px;
  width: 200px;
  background: #FFFFFF url("../../common/openid-16x16.gif") 5px 5px no-repeat;
  padding-left: 25px;
}

/* -------------------------------------------------------------------------- */
/* Custom classes                                                             */
/* -------------------------------------------------------------------------- */

span.text-error,
div.text-error,
p.text-error {
  color: #E00;
  font-weight: bold;
}

span.text-help,
div.text-help,
p.text-help {
  font-size: 0.9em;
}

span.text-info,
div.text-info,
p.text-info {
  font-style: italic;
}

span.text-label,
div.text-label,
p.text-label {
  font-weight: bold;
}

div.panel-buttons {
  text-align: right;
}

div.title {
  vertical-align: bottom;
  font-weight: bold;
  font-size: 2em;
}

div.title img {
  vertical-align: bottom;
}

ul.nostyle,
ul.nostyle li {
  list-style-type: none;
  margin: 0px;
  padding: 0px;
}

div.authLogo {
  text-align: center;
  margin-top: 10px;
}

/* -------------------------------------------------------------------------- */
/* Main layout                                                                */
/* -------------------------------------------------------------------------- */

#layout {
  position: absolute;
  top: 100px;
  right: 0px;
  left: 0px;
  width: 700px;
  height: 340px;
  margin: auto;
}

#content {
  padding: 20px;
  background-color: #F8F8F8;
  color: #000;
  border: 1px solid #EAEAEA;
  border-radius: 10px;
  -moz-border-radius:10px;
  -webkit-border-radius:10px;
}

#content-all {
  padding: 10px;
}

#content-all table {
  margin: 0px;
  padding: 0px;
}

#content-all table tr td {
  vertical-align: top;
}

#content-all-info {
  padding: 10px;
  margin-bottom: 15px;
  background-color: #D5EFD1;
  color: #000;
  border: 1px solid #B5CFB6;
  border-radius: 10px;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
}

#content-all-info2 {
  text-align: center;
  padding: 10px;
  margin-bottom: 15px;
  background-color: #D5EFD1;
  color: #000;
  border: 1px solid #B5CFB6;
  border-radius: 10px;
  -moz-border-radius: 0px 0px 10px 10px;
  -webkit-border-radius: 0px 0px 10px 10px;
}

#content-all-info table.info,
#content-all-info2 table.info {
  width: 100%;
  border-collapse: collapse;
}

#content-all-info table.info th,
#content-all-info2 table.info th {
  background-color: #B5CFB6;
}

#content-all-info table.info th,
#content-all-info2 table.info th
#content-all-info table.info td,
#content-all-info2 table.info td {
  text-align: left;
  border-bottom: 1px dotted #B5CFB6;
  padding: 5px;
}

#content-left {
  position: absolute;
  width: 370px;
  padding: 10px;
}

#content-left h1 {
  font-size: 1.5em;
}

#content-right {
  margin-left: 400px;
  padding: 10px;
  background-color: #D5EFD1;
  color: #000;
  border: 1px solid #B5CFB6;
  border-radius: 10px;
  -moz-border-radius:10px;
  -webkit-border-radius:10px;
}

#content-right h1 {
  font-size: 1.3em;
}

#content-right hr {
  color: #B5CFB6;
  background-color: #B5CFB6;
}

#content-right img {
  vertical-align: top;
}

#content-right a {
  vertical-align: bottom;
  text-decoration: underline;
}

/* -------------------------------------------------------------------------- */
/* Footer layout                                                              */
/* -------------------------------------------------------------------------- */

#layout-footer {
  padding: 5px;
  text-align: center;
}

#layout-footer a {
  text-decoration: none;
  font-weight: bold;
}

#layout-footer a img {
  border: 0px;
}

/* -------------------------------------------------------------------------- */
/* Applications layer                                                         */
/* -------------------------------------------------------------------------- */

.clearfix {
	clear:both;
}

div.category {
	border:1px solid #B5CFB6;
	-moz-border-radius:10px 10px 10px 10px;
	-webkit-border-radius:10px 10px 10px 10px;
	margin:10px 50px;
	padding:0;
	clear:both;
}

div.category h3.catname {
	background-color:#C7DFC3;
	border-radius:10px 10px 0 0;
	-moz-border-radius:10px 10px 0 0;
	-webkit-border-radius:10px 10px 0 0;
	margin:0;
	padding:10px;
}

div.application {
	margin:10px;
	padding:5px;
	float:left;
	width:200px;
	height:70px;
	overflow:hidden;
}

div.application:hover {
	background:#C7DFC3;
}

div.application img.applogo {
	width:32px;
	height:32px;
	float:left;
}

div.application h4.appname {
	text-align:left;
	padding:0 5px;
	margin:0 0 0 40px;
}

div.application p.appdesc {
	text-align:left;
	padding:0 5px;
	margin:0 0 0 40px;
	text-size:8pt;
	color:#000;
	font-style:italic;
}

/* -------------------------------------------------------------------------- */
/* Menu                                                                       */
/* -------------------------------------------------------------------------- */

#menu, #authMenu {
  background-color: #C7DFC3;
  border: 1px solid #B5CFB6;
  border-bottom: 0px;
  border-radius: 10px 10px 0px 0px;
  -moz-border-radius: 10px 10px 0px 0px;
  -webkit-border-radius: 10px 10px 0px 0px;
}

#authMenu {
  border-bottom: 1px solid #B5CFB6;
  border-radius: 10px;
  -moz-border-radius: 10px;
  -webkit-border-radius: 10px;
}

#menu ul, #authMenu ul {
  list-style: none;
  margin: 0px;
  padding: 5px;
}

#menu ul li, #authMenu ul li {
  list-style: none;
  display: inline;
}

#menu ul li a img, #authMenu ul li a img {
  border: none;
  vertical-align: bottom;
  margin-left: 10px;
}

.ui-tabs-hide {
	display:none;
}

