<html>
<body>

<div id="page" style="background:#000;font-family:sans-serif;font-size:12pt;color:#fff;padding:5px 20px;">

<div id="header" style="text-align:right;"><h1>LemonLDAP::NG</h1></div>

<div id="content" style="color:#000;background:#eee;padding:5px 10px;margin:10px 0;">
