/* CSS for skin PASTEL */
/* Shipped with LemonLDAP::NG */

/* Use buttons CSS */
@import url("../../common/buttons.css");

/* Use input icons CSS */
@import url("../../common/inputicons.css");

body {
font-family:Arial, Helvetica, Verdana;
font-size:11pt;
color:#000;
background:#eee url(../images/hatch.gif);
margin:0;
padding:0;
}

h1, h2, h3, h4 {
margin:0;
padding:0 0 5px 0;
}

h3 {
margin:20px;
}

a img,:link img,:visited img {
border:none;
}

a, a:link, a:visited {
text-decoration:none;
color:#336699;
}

#page {
margin:20px 40px;
padding:0;
background:#fff;
border:1px solid #C6D880;
text-align:center;
box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-moz-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-webkit-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

#footer {
clear:both;
padding:5px;
margin:0;
text-align:center;
font-size:10pt;
}

#footer p {
margin:10px;
padding:5px;
background-color:#f5f5f5;
border:1px solid #dedede;
font-size:8pt;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

div.loginlogo {
background:url(../images/lock.png) no-repeat center;
width:200px;
height:250px;
float:left;
}

div.menulogo {
display:none;
}

div.user {
float:right;
text-align:right;
color:#529214;
font-size:10pt;
font-weight:bold;
margin:13px 15px 0 0;
}

div.message {
margin:20px 50px;
padding:0;
display:none;
text-align:left;
box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-moz-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-webkit-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

ul {
list-style:none;
margin:0;
padding:5px;
}

li {
list-style:none;
}

div.message li {
font-size:11pt;
font-weight:bold;
list-style-type:none;
background-repeat:no-repeat;
background-position:0% 60%;
padding:3px 0 3px 30px;
}

.message.negative li {
background-image:url("../../common/warning.png");
}

.message.positive li {
background-image:url("../../common/ok.png");
}

.message.warning li {
background-image:url("../../common/warning_triangle.png");
}

#authMenu ul {
margin:0px 200px -20px 200px;
box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-moz-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-webkit-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

form, div.form {
display:block;
overflow:visible;
padding:0;
margin:10px;
text-align:center;
background-color:#dff4ff;
border:1px solid #c2e1ef;
color:#336699;
box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-moz-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-webkit-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

form.login, form.password, div.form {
margin:40px 200px;
}

form table, div.form table {
border:0;
margin: 0 auto 10px;
}

form th {
text-align: right;
}

form input {
border:1px solid #c2e1ef;
padding:5px;
width: 200px;
}

form input[name=openid_identifier] {
width: 300px;
}

/* JQuery tabs */

.ui-tabs-hide {
display:none;
}

.ui-tabs-nav {
text-align:left;
margin:10px 0;
padding:12px 5px 10px 5px;
background-color:#C6D880;
}

.ui-tabs-nav li{
display:inline;
margin-left:10px;
padding:9px 10px 6px 10px;
font-size:10pt;
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
}

.ui-tabs-nav li a{
color:#529214;
font-weight:bold;
}

li.ui-tabs-active{
background-color:#E6EFC2;
}

/* Application list */

#appslist{
padding: 10px;
}

.clearfix {
clear:both;
}

div.category {
border:1px solid #c2e1ef;
box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-moz-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
-webkit-box-shadow:0 3px 6px rgba(0, 0, 0, 0.25);
border-radius:10px 10px 10px 10px;
-moz-border-radius:10px 10px 10px 10px;
-webkit-border-radius:10px 10px 10px 10px;
margin:10px 50px;
padding:0;
clear:both;
}

div.category h3.catname {
background-color:#dff4ff;
color:#336699;
border-radius:10px 10px 0 0;
-moz-border-radius:10px 10px 0 0;
-webkit-border-radius:10px 10px 0 0;
margin:0;
padding:10px;
}

div.application {
margin:10px;
padding:5px;
float:left;
width:30%;
height:50px;
overflow:hidden;
}

div.application:hover {
background:#eee;
}

div.application img.applogo {
width:32px;
height:32px;
float:left;
}

div.application h4.appname {
text-align:left;
padding:0 5px;
margin:0 0 0 40px;
}

div.application p.appdesc {
text-align:left;
padding:0 5px;
margin:0 0 0 40px;
text-size:8pt;
color:#aaa;
font-style:italic;
}

/* Logout */

#logout .buttons a, #logout .buttons button, #error .buttons a, #error .buttons button{
display:block;
display:inline !important;
float:none;
margin:0;
width:150px;
}

#logout {
margin-bottom:20px;
}

/* Info */

table.info, .info td, .info tr, .info th{
text-align:center;
border-collapse:collapse;
border:1px solid #C2E1EF;
padding:5px 10px;
}

.info th{
background-color:#C2E1EF;
}

p.removeOther{
font-weight:bold;
}

div.link, div.login_info {
text-align:center;
font-weight:bold;
margin:40px 200px;
}

