    <!-- Custom HTML footer -->
    <p>
      <lang en="Service provided by" fr="Ce service est fourni par" />
      <a href="http://lemonldap-ng.org" target="_blank">LemonLDAP::NG</a>,
      <lang en="free software covered by the GPL license" fr="logiciel libre prot&eacute;g&eacute; par la licence GPL" />.
    </p>
    <a href="http://lemonldap-ng.org" target="_blank"><img src="<TMPL_VAR NAME="SKIN_PATH">/<TMPL_VAR NAME="SKIN">/images/logo_lemonldap-ng_400px.png" alt="LemonLDAP::NG logo" width="120" /></a>

