<TMPL_INCLUDE NAME="mail_header.tpl">

<p>
<lang en="Hello" fr="Bonjour" /> $cn,<br />
<br />
<span><img src="cid:arrow:skins/common/bullet_go.png" /></span>
<a href="$url" style="text-decoration:none;color:orange;">
<lang en="Click here to reset your password" fr="Cliquez ici pour réinitialiser votre mot de passe" />
</a>
</p>

<TMPL_INCLUDE NAME="mail_footer.tpl">
