</div>

<div id="footer" style="font-size:9pt;">
<p>
<lang en="This mail was sent automatically" fr="Ceci est un message automatique" /><br />
<lang en="The reset password request was issued from IP" fr="La demande de changement de mot de passe provient de l'IP" />
$ipAddr
</p>
</div>

</div>

</body>
</html>
